package schedframe.resources.devices;

import schedframe.Initializable;
import schedframe.events.Event;
import schedframe.events.EventHandler;
import schedframe.resources.Resource;
import schedframe.resources.ResourceStatus;
import schedframe.resources.ResourceType;
import schedframe.resources.computing.ResourceCharacteristics;
import schedframe.resources.computing.extensions.Extension;
import schedframe.resources.computing.extensions.ExtensionList;
import schedframe.resources.computing.extensions.ExtensionType;
import schedframe.resources.computing.profiles.energy.EnergyExtension;
import schedframe.resources.computing.profiles.energy.airthroughput.ui.AirThroughputInterface;
import schedframe.resources.computing.profiles.energy.power.ui.PowerInterface;
import schedframe.resources.computing.profiles.energy.thermal.ui.ThermalInterface;

public abstract class PhysicalResource implements Resource, Initializable{

	protected String name;
	protected ResourceType type;
	protected String category;
	protected ResourceStatus status;
	protected ResourceCharacteristics resourceCharacteristic;
	protected ExtensionList extensionList;

	
	public abstract void handleEvent(Event event);
	
	public abstract EventHandler getEventHandler();
	
	public String getName() {
		return name;
	}

	public ResourceType getType() {
		return type;
	}

	public String getCategory(){
		return category;
	}
	
	public abstract void setStatus(ResourceStatus newStatus);
	
	public ResourceStatus getStatus() {
		return status;
	}
	
	private Extension getExtension(ExtensionType type){
		if (extensionList != null) {
			for (Extension extension : extensionList) {
				if (extension.getType() == type) {
					return extension;
				}
			}
		}
		return null;
	}
	
	public PowerInterface getPowerInterface(){
		Extension extension = getExtension(ExtensionType.ENERGY_EXTENSION);
		if(extension != null){
			EnergyExtension ee = (EnergyExtension)extension;
			return ee.getPowerInterface();
		}
		return null;
	}
	
	public AirThroughputInterface getAirThroughputInterface(){
		Extension extension = getExtension(ExtensionType.ENERGY_EXTENSION);
		if(extension != null){
			EnergyExtension ee = (EnergyExtension)extension;
			return ee.getAirThroughputInterface();
		}
		return null;
	}
	
	public ThermalInterface getThermalInterface(){
		Extension extension = getExtension(ExtensionType.ENERGY_EXTENSION);
		if(extension != null){
			EnergyExtension ee = (EnergyExtension)extension;
			return ee.getThermalInterface();
		}
		return null;
	}
	
	public ExtensionList getExtensionList() {
		return extensionList;
	}
	
	protected void accept(Extension e){
		extensionList.add(e);
	}

	public ResourceCharacteristics getResourceCharacteristic() {
		return resourceCharacteristic;
	}
	
}
